//  EPLD OCIW BASE ADDRESS DECODER + FIBER INTERFACE
//  For DSP v3 -- WITH 23-STAGE SCRAM/DSCRAM
//  Author              Greg Burley
//  Organization        OCIW
//  Revision Level      3.6
//  Target Device       EPM7256AETC144-5

// Module Declaration
module epld (DQ, D, BA, SS, A, CLK, RESET, RD, WR, AA3, MR, FTXD, FTXCK,
	FRXD, FRXCK, ERXD, ERXCK, ERXFS, BRW, BWR, BRD, SQCK, SQEN, IRQA, IRQB,
	OPT3V, REFCLK, SDATA, RDATA, G0, G1, G2, G3, G4, G5, G6, G7, RXEN, 
	RCNT, TX_LD);

inout	[15:0] DQ;				// Bidirectional backplane data bus
inout	[15:0] D;				// Bidirectional DSP data bus
output	[5:0]  BA;				// Backplane address
output	[11:0] SS;				// Slot select
input	[9:0]  A;				// address bus
input	CLK, RESET;				// DSP clock + global reset
input	RD, WR;					// DSP read + write
input	AA3;					// DSP external peripheral
output	MR;						// reset trigger
output	FTXD, FTXCK;			// optical fiber transmit signals
input	FRXD, FRXCK;			// optical fiber receive signals
output	ERXD, ERXCK, ERXFS;		// wire link serial signals
output	BRW, BWR, BRD;			// backplane R/W + write + read
output	SQCK, SQEN;				// SEQ register clock + enable
output	IRQA, IRQB;				// DSP interupt lines
input	OPT3V;					// optical=1 wire=0
output	REFCLK;					// CDR chip reference clock
output	SDATA, RDATA;			// unscrambled send + receive data
output	G0, G1, G2, G3, G4, G5, G6, G7;
output	RXEN;					// RX register enable
output	[4:0] RCNT;
output	TX_LD;

wire OEa, OEb;
wire [22:0] SCRAM, DSCRAM;
wire [17:0] TSHIFT, TSHIFT_d;
wire [15:0] RSHIFT, RXREG, DMUX;
wire [4:0] TCNT, TCNT_eq;
wire [4:0] RCNT, RCNT_eq;
wire [5:0] INT, INT_eq;
wire [2:0] CNT;
wire TX_LD, RX_LD;
wire RDATA, RDATA_d;
wire DRXD, DRCNT;
wire RXRDY;
wire SCRAM_in;
wire INT_cout, INT_cnt_en, INT_sclr;

//	DQBUS BRIDGES TO BACKPLANE -- DBUS BRIDGES TO DSP
//  A = (COND) ? TRUE: FALSE;
assign DQ = (OEb) ? D : 16'bz;
assign DMUX = (RXEN) ? DQ : RXREG;
assign D = (OEa) ? DMUX : 16'bz;
assign OEa = (~RD & ~AA3);
assign OEb = (~WR & ~AA3 & ((A[9:0]!='H389) & (A[9:0]!='H388)) );
//assign OEb = (~WR & ~AA3 & (A[9:0] != 'H389) );

// 	CLOCK CIRCUIT + TXCLK AND ADCLK OUTPUT
lpm_counter	 cnt(.clock(CLK),.q(CNT));
	defparam cnt.lpm_width=3;
	defparam cnt.lpm_direction="UP";

assign FTXCK = (OPT3V) ? CLK : 1'bz;
assign REFCLK = (OPT3V) ? CNT[2] : 1'bz;	//	CLK/8

//	SCRAMBLER + DESCRAMBLER DESCRIPTION -- 23-STAGE
//	SCRAM.shiftin = SDATA $ SCRAM.q[4] $ SCRAM.q[22];
lpm_shiftreg scram(.clock(CLK), .q(SCRAM), .shiftin (SCRAM_in));
	defparam scram.lpm_width=23;
	defparam scram.lpm_direction="LEFT";

assign SCRAM_in = (SDATA ^ SCRAM[4] ^ SCRAM[22]);
assign FTXD = (OPT3V) ? SCRAM[22] : 1'bz;

lpm_shiftreg dscram(.clock(FRXCK), .q(DSCRAM), .shiftin(FRXD));
	defparam dscram.lpm_width=23;
	defparam dscram.lpm_direction="LEFT";

assign RDATA_d = (FRXD ^ DSCRAM[4] ^ DSCRAM[22]);
dff	u1(.clk(FRXCK), .d(RDATA_d), .q(RDATA));

// 	TX SHIFT REGISTER AT ADDRESS $FF89 USING /SS1 
lpm_shiftreg tshift(.clock(CLK), .data(TSHIFT_d), .q(TSHIFT), .load(TX_LD),
	.shiftin (~RESET), .shiftout(SDATA));
	defparam tshift.lpm_width=18;
	defparam tshift.lpm_direction="RIGHT";

assign TSHIFT_d[17:0] = {~RESET, D[15:0], 1'b1};
dff	u2(.clk(CLK), .d(~WR & ~AA3 & (A[9:0]=='H389)), 
	.q(TX_LD), .clrn(~WR));

// 	TX FRAME_SYNC/BUSY GENERATOR CIRCUIT -- IRQA IS TX DONE
lpm_counter	tcnt(.clock(CLK), .q(TCNT), .sset(TX_LD), .eq(TCNT_eq), 
	.cnt_en(~TCNT_eq[0]));
	defparam tcnt.lpm_width=5;
	defparam tcnt.lpm_direction="DOWN";
	defparam tcnt.lpm_svalue=18;

assign IRQA = ~TCNT_eq[0] | ~RESET;		// IRQA = VCC during reset 

// 	XREG LOADS FROM RX SHIFT REG -- IRQB IS RX READY
lpm_shiftreg rshift(.clock(FRXCK), .q(RSHIFT), .shiftin(RDATA));
	defparam rshift.lpm_width=16;
	defparam rshift.lpm_direction="RIGHT";

lpm_counter	rcnt(.clock(FRXCK), .q(RCNT), .eq(RCNT_eq), .cnt_en(~RCNT_eq[0]),
	.sset(RDATA & RCNT_eq[0]) );
	defparam rcnt.lpm_width=5;
	defparam rcnt.lpm_direction="DOWN";
	defparam rcnt.lpm_svalue=17;

lpm_ff u3(.clock(FRXCK), .data(RSHIFT[15:0]), .q(RXREG[15:0]), .enable(RCNT_eq[1]));
	defparam u3.lpm_width=16;

dffe u4(.clk(FRXCK), .d(IRQB), .q(RXRDY), .ena(DRCNT), .clrn(RXEN));

assign IRQB = ~RCNT_eq[0] & RESET; 		// IRQB = GND during reset 
assign RXEN = ~(~AA3 & (A[9:0]=='H38A));


// 	GRAB + DECODE RESET SIGNAL FROM RX SHIFT REG
dff	u5(.clk(FRXCK), .d(RDATA), .q(DRXD));
dff	u6(.clk(FRXCK), .d(RCNT_eq[2]), .q(DRCNT));

lpm_counter	int (.clock(FRXCK), .q(INT), .eq(INT_eq), .cout(INT_cout),
	.clk_en(DRCNT), .cnt_en(INT_cnt_en), .sclr(INT_sclr), .aclr(~RESET));
	defparam int.lpm_width=6;
	defparam int.lpm_direction="UP";

assign INT_cnt_en = (RDATA & ~DRXD & ~INT_cout);
assign INT_sclr = (DRCNT & (~RDATA | DRXD));

assign MR = (OPT3V & INT_cout) ? ~INT_cout : 1'bz;

//	SEQUENCE REGISTER AND BACKPLANE BUFFER DETAILS --  BRW L=RD H=WR
assign BA[5:0] = A[5:0];
assign BRW = ~WR & ~AA3 & SS[1];
assign BRD = RD;
assign BWR = WR;
assign SQCK = WR;
assign SQEN = ~(~AA3 & (A[9:0]=='H388));

//	SLOT SELECT SIGNALS FOR THE VARIOUS BOARDS
assign SS[0] = ~((A[9:0] >= 'H380) & (A[9:0] <= 'H387) & ~AA3);
assign SS[1] = ~((A[9:0] >= 'H388) & (A[9:0] <= 'H38F) & ~AA3);

assign SS[2] = ~((A[9:0] >= 'H000) & (A[9:0] <= 'H01F) & ~AA3);
assign SS[3] = ~((A[9:0] >= 'H020) & (A[9:0] <= 'H03F) & ~AA3);
assign SS[8] = ~((A[9:0] >= 'H040) & (A[9:0] <= 'H05F) & ~AA3);
assign SS[9] = ~((A[9:0] >= 'H060) & (A[9:0] <= 'H07F) & ~AA3);

assign SS[4] = ~((A[9:0] >= 'H100) & (A[9:0] <= 'H11F) & ~AA3);
assign SS[5] = ~((A[9:0] >= 'H140) & (A[9:0] <= 'H15F) & ~AA3);
assign SS[6] = ~((A[9:0] >= 'H180) & (A[9:0] <= 'H19F) & ~AA3);
assign SS[7] = ~((A[9:0] >= 'H1C0) & (A[9:0] <= 'H1DF) & ~AA3);

assign SS[10] = 1'bz;
assign SS[11] = 1'bz;

// 	NON-FUNCTIONING SIGNALS FOR DSP ESSI PORT -- FUTURE USE
assign ERXD = 1'bz;
assign ERXFS = 1'bz;
assign ERXCK = 1'bz;

// 	NON-FUNCTIONING SPARE PINS -- FUTURE USE
assign G0 = 1'bz;
assign G1 = 1'bz;
assign G2 = 1'bz;
assign G3 = RXRDY;
assign G4 = RXEN;
assign G6 = RDATA;
assign G5 = SDATA;
assign G7 = 1'bz;

//	SCRAMBLER + DESCRAMBLER DESCRIPTIONS
//	SCRAM.shiftin = SDATA ^ SCRAM.q[3] ^ SCRAM.q[8];  (9-STAGE)
//	SCRAM.shiftin = SDATA ^ SCRAM.q[1] ^ SCRAM.q[14]; (15-STAGE)
//	SCRAM.shiftin = SDATA ^ SCRAM.q[4] ^ SCRAM.q[22]; (23-STAGE)
//	SCRAM.shiftin = SDATA ^ SCRAM.q[2] ^ SCRAM.q[30]; (31-STAGE)


//	BOARD  		SLOT 	DECODED ADDRESS			DEVICE
//	======= 	=====	==================		=======
//	POWER  		/SS0	$FFFF80					DAC
//	DSP 		/SS1	$FFFF88					SEQREG
//						$FFFF89					HSS TX
//						$FFFF8A					HSS RX
//	SIGNAL		/SS2	$FFFC00					ADC_A
//	PROC				$FFFC01					ADC_B
//						$FFFC10 -- $FFFC17		DAC
//						$FFFC18 -- $FFFC1F		DAC
//	SIG_PROC	/SS3	$FFFC20					ADC_C
//						$FFFC21					ADC_D
//						$FFFC30 -- $FFFC37		DAC
//						$FFFC38 -- $FFFC3F		DAC
//	SIG_PROC	/SS8	$FFFC40					ADC_E
//						$FFFC41					ADC_F
//						$FFFC50 -- $FFFC57		DAC
//						$FFFC58 -- $FFFC5F		DAC
//	SIG_PROC	/SS9	$FFFC60					ADC_G
//						$FFFC61					ADC_H
//						$FFFC70 -- $FFFC77		DAC
//						$FFFC78 -- $FFFC7F		DAC

//	CLK_DRV		/SS4	$FFFD00 -- $FFFD07		DAC
//						$FFFD08 -- $FFFD0F		DAC
//						$FFFD10 -- $FFFD17		DAC
//						$FFFD18 -- $FFFD1F		DAC
//	CLK_DRV		/SS5	$FFFD40 -- $FFFD47		DAC
//						$FFFD48 -- $FFFD4F		DAC
//						$FFFD40 -- $FFFD47		DAC
//						$FFFD48 -- $FFFD4F		DAC
//	CLK_DRV		/SS6	$FFFD80 -- $FFFD87		DAC
//						$FFFD88 -- $FFFD8F		DAC
//						$FFFD90 -- $FFFD97		DAC
//						$FFFD98 -- $FFFD9F		DAC
//	CLK_DRV		/SS7	$FFFDC0 -- $FFFDC7		DAC
//						$FFFDC8 -- $FFFDCF		DAC
//						$FFFDD0 -- $FFFDD7		DAC
//						$FFFDD8 -- $FFFDDF		DAC

endmodule

